﻿'use strict';

var chatUtils = (function () {
    var ChatCallOptions =
    {
        AUDIO: 1,
        AUDIO_AND_VIDEO: 2
    };

    var CallAutomationModeEnum =
    {
        PICK_UP: 0,
        CALL: 1
    };

    var ChatAutomationEventConditionFieldEnum =
    {
        VISITOR_IS_ON_WEB: 1,
        VISITOR_IS_CHATTING_WITH_OPERATOR: 2,
        VISITOR_SENT_MESSAGE: 3,
        NUMBER_OF_VISITS: 4
    };

    var ChatAutomationEventConditionOperatorEnum =
    {
        IS: 1,
        IS_NOT: 2,
        IS_GREATER_THAN: 3,
        IS_LESS_THAN: 4,
        IS_BEFORE: 5,
        IS_AFTER: 6,
        CONTAINS: 7,
        BEGINS_WITH: 8,
        ENDS_WITH: 9,
        DOES_NOT_CONTAIN: 10,
        YES: 11,
        NO: 12
    };

    var ChatSessionOfferTypeEnum = {
        Add: 0,
        PassOn: 1,
        properties: {
            0: { name: "Přidat" },
            1: { name: "Předat" }
        }
    };

    var ChatUserStatusEnum =
    {
        AVAILABLE: 0,
        BUSY: 1,
        AWAY: 2
    };

    var ChatMessageTypeEnum = {
        COMMON: 0,
        JOIN: 1,
        ADD: 2,
        PASS_ON: 3,
        OFFER: 4,
        REFUSE: 5,
        CLOSE: 6,
        ENTER: 7,
        LEAVE: 8,
        CONNECTION_LOST: 9,
        ATTACHMENTS: 10,
        COMMON_INCOMPLETE: 11,
        AUTOMATION: 12,
        START_CALL: 13,
        END_CALL: 14,
        OPTIONS: 15
    };

    var CustomFormFieldTypeEnum = {
        STRING: 1,
        BOOLEAN: 2
    };

    var ChatUserSessionCallStateEnum = {
        NONE: 0,
        RINGING: 1,
        CALLING: 2,
        properties: {
            0: { name: "Žádný" },
            1: { name: "Zvonění" },
            2: { name: "Volání" }
        }
    };

    var ChatUserVisibilityEnum = {
        VISIBLE_TO_EVERYBODY: 0,
        INVISIBLE_TO_GUESTS: 1,
        INVISIBLE_TO_EVERYBODY: 2,
        OFFLINE: 3,
        properties: {
            0: { name: "Viditelný" },
            1: { name: "Neviditelný pro hosty" },
            2: { name: "Neviditelný pro všechny" },
            3: { name: "Offline" }
        }
    };

    var ChatProfileAutomationEventEventTypeEnum = {
        LoadChatWidget: 1,
        SendMessageToChat: 2,
        properties: {
            1: { name: "Nahrání chat widgetu" },
            2: { name: "Odeslání první zprávy" }
        }
    };

    var chatMessageTypeEnum2MessageTemplateName = function (chatMessageType) {
        switch (chatMessageType) {
            case ChatMessageTypeEnum.COMMON:
            case ChatMessageTypeEnum.COMMON_INCOMPLETE:
            case ChatMessageTypeEnum.AUTOMATION:
            case ChatMessageTypeEnum.OPTIONS:
            case ChatMessageTypeEnum.ATTACHMENTS: return "bubbleMsgTemplate";
            default: return "eventTemplate";
        }
    };

    var chatMessengerMessageTypeEnum2MessageTemplateName = function (chatMessageType) {
        switch (chatMessageType) {
            case ChatMessageTypeEnum.COMMON:
            case ChatMessageTypeEnum.COMMON_INCOMPLETE:
            case ChatMessageTypeEnum.ATTACHMENTS: return "commonAndAttachmentsMessengerMsgTemplate";
            default: return "messengerEventTemplate";
        }
    };

    var chatMessageTypeEnum2EventTemplateName = function (chatMessageType) {
        switch (chatMessageType) {
            case ChatMessageTypeEnum.JOIN: return "joinMsgTemplate";
            case ChatMessageTypeEnum.ADD: return "addMsgTemplate";
            case ChatMessageTypeEnum.PASS_ON: return "passOnMsgTemplate";
            case ChatMessageTypeEnum.OFFER: return "offerMsgTemplate";
            case ChatMessageTypeEnum.REFUSE: return "refuseMsgTemplate";
            case ChatMessageTypeEnum.CLOSE: return "closeMsgTemplate";
            case ChatMessageTypeEnum.ENTER: return "enterMsgTemplate";
            case ChatMessageTypeEnum.LEAVE: return "leaveMsgTemplate";
            case ChatMessageTypeEnum.START_CALL: return "startCallTemplate";
            case ChatMessageTypeEnum.END_CALL: return "endCallTemplate";
            case ChatMessageTypeEnum.CONNECTION_LOST: return "connectionLostMsgTemplate";
            default: throw "Unknown event type.";
        }
    };

    var chatMessengerMessageTypeEnum2EventTemplateName = function (chatMessageType) {
        switch (chatMessageType) {
            case ChatMessageTypeEnum.START_CALL: return "messengerStartCallTemplate";
            case ChatMessageTypeEnum.END_CALL: return "messengerEndCallTemplate";
            default: throw "Unknown event type.";
        }
    };

    var chatUserStatusEnum2Color = function (status) {
        switch (status) {
            case ChatUserStatusEnum.AVAILABLE: return "darkgreen";
            case ChatUserStatusEnum.BUSY: return "darkred";
            case ChatUserStatusEnum.AWAY: return "darkorange";
            default: throw "Unknown status value: " + status + ".";
        }
    };

    var chatUserStatusEnum2Name = function (status) {
        switch (status) {
            case ChatUserStatusEnum.AVAILABLE: return Localization_Chat_UserStatus_Available;
            case ChatUserStatusEnum.BUSY: return Localization_Chat_UserStatus_Busy;
            case ChatUserStatusEnum.AWAY: return Localization_Chat_UserStatus_Away;
            default: throw "Unknown nn status value: " + status + ".";
        }
    };

    var buildChatSessionUrl = function (basePath, params, jQueryArg) {
        var jQueryLocal = jQueryArg || jQuery;
        basePath = basePath || RQ_URL + "/";

        if (params) {
            return basePath + 'Chat/ChatGuestSession/?' + jQueryLocal.param(params);
        }
    };

    var detectClientMedia = function () {
        return new Promise(function (resolve, reject) {
            if (navigator.mediaDevices === undefined)
            {
                resolve();
                return;
            }


            navigator.mediaDevices.enumerateDevices().then(function (deviceInfos) {
                var result = {
                    isMicrophoneAvailable: false,
                    isCameraAvailable: false
                };

                for (var i = 0; i !== deviceInfos.length; ++i) {
                    var deviceInfo = deviceInfos[i];
                    if (deviceInfo.kind === 'audioinput') {
                        result.isMicrophoneAvailable = true;
                    } else if (deviceInfo.kind === 'videoinput') {
                        result.isCameraAvailable = true;
                    }
                }

                resolve(result);
            }).catch(function (error) {
                reject(error);
            });
        });
    };

    function numSizeToTextSize(numSize) {
        if (typeof numSize !== 'number')
            numSize = parseInt(numSize);

        return isNaN(numSize) ? null : numSize < 1024 ? (numSize + " B") : (numSize < 1048576 ? ((Math.round(numSize / 1024)) + " kB") : ((Math.round(numSize / 1048576)) + " MB"));
    }

    function runAsynchronously(handler, runLockerName) {
        if (typeof handler !== "function")
            return;

        //var runLockerName = "runLocker_" + randomString(6, 'a');
        chatUtils[runLockerName] = true;
        setTimeout(function () {
            if (runLockerName in chatUtils && chatUtils[runLockerName]) {
                delete chatUtils[runLockerName];
                handler();
            }
        }, 0);
    }

    var SESSION_ID_LENGTH = 26;
    var GUEST_NAME_LENGTH = 10;
    var SESSION_COOKIE_NAME = "ChatSessionR2";
    var GUEST_COOKIE_NAME = "ChatGuestR2";
    var GEO_COOKIE_NAME = "ChatGeoR2";
    var NEW_SESSION_INACTIVITY_DURATION_MS = 2592000000; //30 days

    function getSessionStorage(chatProfileKey) {
        let sessionStorage = $.cookie(SESSION_COOKIE_NAME + "_" + chatProfileKey);
        sessionStorage = sessionStorage ? JSON.parse(sessionStorage) : null;
        let now = Date.now();
        let sessionId = sessionStorage === null || now - sessionStorage.date > NEW_SESSION_INACTIVITY_DURATION_MS ? randomString(SESSION_ID_LENGTH, 'a') : sessionStorage.id;
        let visitsCount = sessionStorage === null ? 1 : now - sessionStorage.date > NEW_SESSION_INACTIVITY_DURATION_MS ? sessionStorage.visitsCount + 1 : sessionStorage.visitsCount;
        let triggeredAutomations = sessionStorage === null || now - sessionStorage.date > NEW_SESSION_INACTIVITY_DURATION_MS ? {} : sessionStorage.triggeredAutomations;
        let highlighterClosed = sessionStorage === null || now - sessionStorage.date > NEW_SESSION_INACTIVITY_DURATION_MS ? false : sessionStorage.highlighterClosed;

        sessionStorage = { id: sessionId, date: now, visitsCount: visitsCount, triggeredAutomations: triggeredAutomations, highlighterClosed: highlighterClosed };
        $.cookie(SESSION_COOKIE_NAME + "_" + chatProfileKey, JSON.stringify(sessionStorage), { path: '/', expires: 1000 });
        return sessionStorage;
    }

    function sessionStorageSetTriggeredAutomation(chatProfileKey, automationId, triggerDate) {
        let sessionStorage = $.cookie(SESSION_COOKIE_NAME + "_" + chatProfileKey);
        sessionStorage = sessionStorage ? JSON.parse(sessionStorage) : null;
        if (sessionStorage) {
            sessionStorage.triggeredAutomations[automationId.toString()] = triggerDate;
            $.cookie(SESSION_COOKIE_NAME + "_" + chatProfileKey, JSON.stringify(sessionStorage), { path: '/', expires: 1000 });
        }
    }

    function resetSession(chatProfileKey) {
        let sessionStorage = $.cookie(SESSION_COOKIE_NAME + "_" + chatProfileKey);
        sessionStorage = sessionStorage ? JSON.parse(sessionStorage) : null;
        if (sessionStorage) {
            sessionStorage.id = randomString(SESSION_ID_LENGTH, 'a');
            $.cookie(SESSION_COOKIE_NAME + "_" + chatProfileKey, JSON.stringify(sessionStorage), { path: '/', expires: 1000 });
        }
    }

    function closeHighlighter(chatProfileKey) {
        let sessionStorage = $.cookie(SESSION_COOKIE_NAME + "_" + chatProfileKey);
        sessionStorage = sessionStorage ? JSON.parse(sessionStorage) : null;
        if (sessionStorage) {
            sessionStorage.highlighterClosed = true;
            $.cookie(SESSION_COOKIE_NAME + "_" + chatProfileKey, JSON.stringify(sessionStorage), { path: '/', expires: 1000 });
        }
    }

    function getGuestProviderKey() {
        var guestStorage = $.cookie(GUEST_COOKIE_NAME);
        if (!guestStorage) {
            guestStorage = "guest " + randomString(GUEST_NAME_LENGTH, '#');
            $.cookie(GUEST_COOKIE_NAME, guestStorage, { path: '/', expires: 1000 });
        }

        return guestStorage;
    }

    function getIEVersion() {
        var sAgent = window.navigator.userAgent;
        var Idx = sAgent.indexOf("MSIE");

        // If IE, return version number.
        if (Idx > 0)
            return parseInt(sAgent.substring(Idx + 5, sAgent.indexOf(".", Idx)));

        // If IE 11 then look for Updated user agent string.
        else if (!!navigator.userAgent.match(/Trident\/7\./))
            return 11;

        else
            return 0; //It is not IE
    }

    function isToday(someDate) {
        someDate = new Date(someDate);
        let today = new Date();
        return someDate.getDate() === today.getDate() &&
            someDate.getMonth() === today.getMonth() &&
            someDate.getFullYear() === today.getFullYear();
    }

    function randomString(length, chars) {
        var mask = '';
        if (chars.indexOf('a') > -1) mask += 'abcdefghijklmnopqrstuvwxyz';
        if (chars.indexOf('A') > -1) mask += 'ABCDEFGHIJKLMNOPQRSTUVWXYZ';
        if (chars.indexOf('#') > -1) mask += '0123456789';
        if (chars.indexOf('!') > -1) mask += '~`!@#$%^&*()_+-={}[]:";\'<>?,./|\\';
        var result = '';
        for (var i = length; i > 0; --i) result += mask[Math.round(Math.random() * (mask.length - 1))];
        return result;
    }

    function loadGeoFromStorage() {
        return $.cookie(GEO_COOKIE_NAME);
    }

    function saveGeoToStorage(geo) {
        var geoStr = JSON.stringify(geo);
        $.cookie(GEO_COOKIE_NAME, geoStr, { path: '/' + window.location.pathname.substr(1, window.location.pathname.lastIndexOf('/') - 1) });
    }

    function requestAndSaveGeo() {
        return new Promise(function (resolve, reject) {
            $.getJSON(window.location.protocol + '//ipinfo.io/geo', function (geo) {
                saveGeoToStorage(geo);
                resolve(geo);
            }).fail(function () {
                reject();
            });
        });
    }

    function chatItemIsBeyondChatProfileLimit (chatItem, chatItemsOfChatProfile) {
        let chatProfile = chatItem.chatProfile;
        if (chatProfile && chatProfile.MaxChatsPerOperator !== null && chatItem.chatProfileDate()) {
            if (chatItemsOfChatProfile && chatItemsOfChatProfile.length >= chatProfile.MaxChatsPerOperator) {
                var res = chatItem.chatProfileDate() > chatItemsOfChatProfile[chatProfile.MaxChatsPerOperator - 1].chatProfileDate();
                return res;
            }
        }

        return false;
    }

    function getChatMessageHeaderTextColor(textColor, backgroundColor) {
        var tinyTextColor = tinycolor(textColor);
        if (tinyTextColor.getBrightness() > tinycolor(backgroundColor).getBrightness())
            return tinyTextColor.darken(30);

        return tinyTextColor.brighten(30);
    }

    function callPrerequisitesToText(callPrerequisites, audioNotVideo) {
        let res = [];

        if (audioNotVideo === true || audioNotVideo === null) {
            if (callPrerequisites.audio & callPrerequisites.masks.localMicrophone)
                res.push(Localization_Chat_LocalMicrophone);

            if (callPrerequisites.audio & callPrerequisites.masks.oppositeMicrophone)
                res.push(Localization_Chat_OppositeMicrophone);

            if (callPrerequisites.audio & callPrerequisites.masks.audioCompatible)
                res.push(Localization_Chat_AudioCompatible);
        }

        if (audioNotVideo === false || audioNotVideo === null) {
            if (callPrerequisites.video & callPrerequisites.masks.localCamera)
                res.push(Localization_Chat_LocalCamera);

            if (callPrerequisites.video & callPrerequisites.masks.oppositeCamera)
                res.push(Localization_Chat_OppositeCamera);

            if (callPrerequisites.video & callPrerequisites.masks.videoCompatible)
                res.push(Localization_Chat_VideoCompatible);
        }

        if (callPrerequisites.security & callPrerequisites.masks.localSecuredConnection)
            res.push(Localization_Chat_LocalSecuredConnection);

        if (callPrerequisites.security & callPrerequisites.masks.oppositeSecuredConnection)
            res.push(Localization_Chat_OppositeSecuredConnection);

        return res.join('\n');
    }

    function addVisibilityChangeListener(visibleHandler, notVisibleHandler) {
        var hidden, visibilityChange;
        if (typeof document.hidden !== "undefined") { // Opera 12.10 and Firefox 18 and later support 
            hidden = "hidden";
            visibilityChange = "visibilitychange";
        } else if (typeof document.msHidden !== "undefined") {
            hidden = "msHidden";
            visibilityChange = "msvisibilitychange";
        } else if (typeof document.webkitHidden !== "undefined") {
            hidden = "webkitHidden";
            visibilityChange = "webkitvisibilitychange";
        }

        document.addEventListener(visibilityChange, function () {
            if (document[hidden]) {
                if (typeof notVisibleHandler === 'function')
                    notVisibleHandler();
            }
            else {
                if (typeof visibleHandler === 'function')
                    visibleHandler();
            }
        }, false);
    }

    return {
        ChatCallOptions: ChatCallOptions,
        CallAutomationModeEnum: CallAutomationModeEnum,
        ChatAutomationEventConditionOperatorEnum: ChatAutomationEventConditionOperatorEnum,
        ChatAutomationEventConditionFieldEnum: ChatAutomationEventConditionFieldEnum,
        ChatSessionOfferTypeEnum: ChatSessionOfferTypeEnum,
        ChatMessageTypeEnum: ChatMessageTypeEnum,
        CustomFormFieldTypeEnum: CustomFormFieldTypeEnum,
        ChatUserSessionCallStateEnum: ChatUserSessionCallStateEnum,
        ChatUserVisibilityEnum: ChatUserVisibilityEnum,
        ChatUserStatusEnum: ChatUserStatusEnum,
        ChatProfileAutomationEventEventTypeEnum: ChatProfileAutomationEventEventTypeEnum,
        chatMessageTypeEnum2MessageTemplateName: chatMessageTypeEnum2MessageTemplateName,
        chatMessageTypeEnum2EventTemplateName: chatMessageTypeEnum2EventTemplateName,
        chatMessengerMessageTypeEnum2MessageTemplateName: chatMessengerMessageTypeEnum2MessageTemplateName,
        chatMessengerMessageTypeEnum2EventTemplateName: chatMessengerMessageTypeEnum2EventTemplateName,
        buildChatSessionUrl: buildChatSessionUrl,
        detectClientMedia: detectClientMedia,
        numSizeToTextSize: numSizeToTextSize,
        getSessionStorage: getSessionStorage,
        resetSession: resetSession,
        closeHighlighter: closeHighlighter,
        sessionStorageSetTriggeredAutomation: sessionStorageSetTriggeredAutomation,
        getGuestProviderKey: getGuestProviderKey,
        loadGeoFromStorage: loadGeoFromStorage,
        requestAndSaveGeo: requestAndSaveGeo,
        getChatMessageHeaderTextColor: getChatMessageHeaderTextColor,
        chatUserStatusEnum2Color: chatUserStatusEnum2Color,
        chatUserStatusEnum2Name: chatUserStatusEnum2Name,
        chatItemIsBeyondChatProfileLimit: chatItemIsBeyondChatProfileLimit,
        getIEVersion: getIEVersion,
        callPrerequisitesToText: callPrerequisitesToText,
        isToday: isToday,
        addVisibilityChangeListener: addVisibilityChangeListener,
        runAsynchronously: runAsynchronously
    };
})();
